<?php

declare(strict_types=1);

namespace Drupal\external_roles;

use Drupal\user\UserInterface;

/**
 * Provides an external roles user wrapper.
 */
class ExternalRolesUser {

  /**
   * Constructs a new ExternalRolesUser object.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user object to be wrapped.
   */
  public function __construct(
    protected UserInterface $user,
  ) {}

  /**
   * Determines whether a user has a certain external role.
   *
   * @param string $role
   *   The role to check.
   *
   * @return bool
   *   TRUE if the user has the external role, otherwise FALSE.
   */
  public function hasExternalRole(string $role): bool {
    return in_array($role, $this->getExternalRoles(), TRUE);
  }

  /**
   * Gets the external roles of a user.
   *
   * @return string[]
   *   An array of all external roles.
   */
  public function getExternalRoles(): array {
    if ($this->user->hasField('external_roles')) {
      return array_column($this->user->get('external_roles')->getValue(), 'value');
    }
    return [];
  }

  /**
   * Adds an external role to a user.
   *
   * @param string $role
   *   The role to add.
   *
   * @return $this
   */
  public function addExternalRole(string $role): static {
    if ($this->user->hasField('external_roles')) {
      $roles = $this->getExternalRoles();
      $roles[] = $role;
      $this->user->set('external_roles', array_unique($roles));
    }
    return $this;
  }

  /**
   * Removes an external role from a user.
   *
   * @param string $role
   *   The role to remove.
   *
   * @return $this
   */
  public function removeExternalRole(string $role): static {
    if ($this->user->hasField('external_roles')) {
      $this->user->set('external_roles', array_diff($this->getExternalRoles(), [$role]));
    }
    return $this;
  }

  /**
   * Resets the external roles of a user.
   *
   * @return $this
   */
  public function resetExternalRoles(): static {
    if ($this->user->hasField('external_roles')) {
      $this->user->set('external_roles', []);
    }
    return $this;
  }

}
