<?php

declare(strict_types=1);

namespace Drupal\external_roles\Service;

use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountInterface;
use Drupal\external_roles\ExternalRolesRepositoryInterface;

/**
 * Provides the external roles service.
 */
class ExternalRoles implements ExternalRolesInterface {

  /**
   * The user external roles keyed by user ID.
   */
  protected array $externalRoles = [];

  /**
   * Constructs a new ExternalRoles service object.
   */
  public function __construct(
    protected Connection $database,
    protected ExternalRolesRepositoryInterface $repository,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getExternalRoles(AccountInterface $account): array {

    // Cache external roles for the account statically to optimize performance,
    // as this method may be frequently invoked when resolving cache contexts.
    // Ensure roles are distinguished by account ID, since roles for different
    // users may be resolved within a single request.
    if (!isset($this->externalRoles[$account->id()])) {

      // This method may be executed before the module is fully installed.
      // Thus, it is necessary to verify the existence of the respective table.
      if ($this->database->schema()->tableExists('user__external_roles')) {
        $roles = $this->database->select('user__external_roles', 'r')
          ->fields('r', ['external_roles_value'])
          ->condition('r.entity_id', $account->id())
          ->orderBy('external_roles_value')
          ->execute()?->fetchCol() ?? [];
      }

      $this->externalRoles[$account->id()] = $roles ?? [];
    }

    return $this->externalRoles[$account->id()];
  }

  /**
   * {@inheritdoc}
   */
  public function getPermissions(AccountInterface $account): array {
    $permissions = [];
    foreach ($this->getExternalRoles($account) as $external_role) {
      $permissions[] = $this->repository->getPermissionsByRole($external_role);
    }
    $permissions = array_merge(...$permissions);
    return array_values(array_unique(array_filter($permissions)));
  }

}
