<?php

declare(strict_types=1);

namespace Drupal\Tests\external_roles\Kernel;

use Drupal\external_roles\ExternalRolesUser;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;

/**
 * Provides tests for the external roles user wrapper.
 *
 * @coversDefaultClass \Drupal\external_roles\ExternalRolesUser
 * @group external_roles
 */
class ExternalRolesUserTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'external_roles',
    'user',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
  }

  /**
   * Tests the external roles user wrapper.
   *
   * @covers::__construct
   * @covers::hasExternalRole
   * @covers::getExternalRoles
   * @covers::addExternalRole
   * @covers::removeExternalRole
   * @covers::resetExternalRoles
   */
  public function testExternalRolesUser(): void {

    // Create a new user and wrap it with the external roles user object.
    $user = $this->createUser();
    $this->assertNotFalse($user);
    $external_roles_user = new ExternalRolesUser($user);
    $this->assertEmpty($external_roles_user->getExternalRoles());

    // It is possible to add an external role to the user.
    $external_roles_user->addExternalRole('alpha');
    $this->assertEquals(['alpha'], $external_roles_user->getExternalRoles());
    $this->assertTrue($external_roles_user->hasExternalRole('alpha'));
    $user->save();

    // The external role persists after saving the user.
    $user = User::load($user->id());
    $this->assertNotNull($user);
    $external_roles_user = new ExternalRolesUser($user);
    $this->assertEquals(['alpha'], $external_roles_user->getExternalRoles());
    $this->assertTrue($external_roles_user->hasExternalRole('alpha'));

    // The external role can be removed.
    $external_roles_user->removeExternalRole('alpha');
    $this->assertEmpty($external_roles_user->getExternalRoles());
    $this->assertFalse($external_roles_user->hasExternalRole('alpha'));

    // Multiple roles can be added and removed without caring about duplicates.
    $external_roles_user->addExternalRole('alpha')
      ->addExternalRole('alpha')
      ->addExternalRole('beta')
      ->addExternalRole('gamma')
      ->addExternalRole('beta');
    $this->assertEquals(['alpha', 'beta', 'gamma'], $external_roles_user->getExternalRoles());
    $this->assertTrue($external_roles_user->hasExternalRole('alpha'));
    $this->assertTrue($external_roles_user->hasExternalRole('beta'));
    $this->assertTrue($external_roles_user->hasExternalRole('gamma'));
    $external_roles_user->removeExternalRole('beta')
      ->addExternalRole('gamma')
      ->removeExternalRole('gamma');
    $this->assertEquals(['alpha'], $external_roles_user->getExternalRoles());
    $this->assertTrue($external_roles_user->hasExternalRole('alpha'));
    $this->assertFalse($external_roles_user->hasExternalRole('beta'));
    $this->assertFalse($external_roles_user->hasExternalRole('gamma'));

    // The external roles may be reset.
    $external_roles_user->addExternalRole('alpha')
      ->addExternalRole('beta')
      ->removeExternalRole('gamma')
      ->addExternalRole('gamma');
    $external_roles_user->resetExternalRoles();
    $this->assertEmpty($external_roles_user->getExternalRoles());

    // The external roles field may be missing.
    $user = $this->prophesize(UserInterface::class);
    $user->hasField('external_roles')->willReturn(FALSE);
    $external_roles_user = new ExternalRolesUser($user->reveal());
    $this->assertEmpty($external_roles_user->getExternalRoles());
  }

}
