<?php

declare(strict_types=1);

namespace Drupal\Tests\external_roles\Unit;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\Context\CacheContextsManager;
use Drupal\Core\Session\AccessPolicyInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\CalculatedPermissionsItem;
use Drupal\Core\Session\RefinableCalculatedPermissions;
use Drupal\external_roles\ExternalRolesAccessPolicy;
use Drupal\external_roles\Service\ExternalRolesInterface;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Prophecy\Prophecy\ObjectProphecy;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides tests for the external roles access policy.
 *
 * @coversDefaultClass \Drupal\external_roles\ExternalRolesAccessPolicy
 * @group external_roles
 */
class ExternalRolesAccessPolicyTest extends UnitTestCase {

  /**
   * The access policy to test.
   */
  protected ExternalRolesAccessPolicy $accessPolicy;

  /**
   * The mocked external roles service.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy<ExternalRolesInterface>
   */
  protected ObjectProphecy $externalRoles;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {

    parent::setUp();

    $this->externalRoles = $this->prophesize(ExternalRolesInterface::class);
    $this->accessPolicy = new ExternalRolesAccessPolicy($this->externalRoles->reveal());

    $cache_context_manager = $this->prophesize(CacheContextsManager::class);
    $cache_context_manager->assertValidTokens(Argument::any())->willReturn(TRUE);

    $container = $this->prophesize(ContainerInterface::class);
    $container->get('cache_contexts_manager')->willReturn($cache_context_manager->reveal());
    \Drupal::setContainer($container->reveal());
  }

  /**
   * Tests the applies method.
   *
   * @covers ::__construct
   * @covers ::applies
   */
  public function testApplies(): void {
    $this->assertTrue($this->accessPolicy->applies(AccessPolicyInterface::SCOPE_DRUPAL));
    $this->assertFalse($this->accessPolicy->applies('another scope'));
    $this->assertFalse($this->accessPolicy->applies($this->randomString()));
  }

  /**
   * Tests the calculatePermissions method.
   *
   * @param array $permissions
   *   The permissions mocked for the account.
   *
   * @covers ::calculatePermissions
   * @dataProvider calculatePermissionsProvider
   */
  public function testCalculatePermissions(array $permissions): void {

    $account = $this->prophesize(AccountInterface::class);
    $this->externalRoles->getPermissions($account->reveal())->willReturn($permissions);
    $this->externalRoles->reveal();

    $calculated_permissions = $this->accessPolicy->calculatePermissions($account->reveal(), AccessPolicyInterface::SCOPE_DRUPAL);

    $this->assertCount(1, $calculated_permissions->getItems(), 'Only one calculated permissions item was added.');
    $this->assertNotFalse($calculated_permissions->getItem());
    $this->assertSame($permissions, $calculated_permissions->getItem()->getPermissions());

    $this->assertSame([], $calculated_permissions->getCacheTags());
    $this->assertSame(['user.external_roles'], $calculated_permissions->getCacheContexts());
    $this->assertSame(Cache::PERMANENT, $calculated_permissions->getCacheMaxAge());
  }

  /**
   * Data provider for testCalculatePermissions.
   *
   * @return array
   *   A list of test scenarios.
   */
  public static function calculatePermissionsProvider(): array {

    $cases['no-permissions'] = [
      'permissions' => [],
    ];

    $cases['one-permission'] = [
      'permissions' => [
        'access foo',
      ],
    ];

    $cases['multiple-permissions'] = [
      'permissions' => [
        'access foo',
        'access bar',
        'access baz',
      ],
    ];

    return $cases;
  }

  /**
   * Tests the alterPermissions method.
   *
   * @covers ::alterPermissions
   */
  public function testAlterPermissions(): void {

    $account = $this->prophesize(AccountInterface::class);

    $calculated_permissions = new RefinableCalculatedPermissions();
    $calculated_permissions->addItem(new CalculatedPermissionsItem(['access foo']));
    $calculated_permissions->addCacheTags(['bar']);
    $calculated_permissions->addCacheContexts(['baz']);

    $this->accessPolicy->alterPermissions($account->reveal(), AccessPolicyInterface::SCOPE_DRUPAL, $calculated_permissions);
    $this->assertNotFalse($calculated_permissions->getItem());
    $this->assertSame(['access foo'], $calculated_permissions->getItem()->getPermissions());
    $this->assertSame(['bar'], $calculated_permissions->getCacheTags());
    $this->assertSame(['baz'], $calculated_permissions->getCacheContexts());
  }

  /**
   * Tests the getPersistentCacheContexts method.
   *
   * @covers ::getPersistentCacheContexts
   */
  public function testGetPersistentCacheContexts(): void {
    $this->assertSame(['user.external_roles'], $this->accessPolicy->getPersistentCacheContexts());
  }

}
