<?php

declare(strict_types=1);

namespace Drupal\Tests\external_roles\Unit;

use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Cache\Context\CacheContextsManager;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\external_roles\Cache\Context\ExternalRolesCacheContext;
use Drupal\external_roles\Service\ExternalRolesInterface;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;

/**
 * Provides tests for the external roles cache context.
 *
 * @coversDefaultClass \Drupal\external_roles\Cache\Context\ExternalRolesCacheContext
 * @group external_roles
 */
class ExternalRolesCacheContextTest extends UnitTestCase {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $cache_contexts_manager = $this->prophesize(CacheContextsManager::class);
    $cache_contexts_manager->assertValidTokens(['user.external_roles'])->willReturn(TRUE);
    $container = new ContainerBuilder();
    $container->set('cache_contexts_manager', $cache_contexts_manager);
    \Drupal::setContainer($container);
  }

  /**
   * Tests the getLabel method.
   *
   * @covers ::__construct
   * @covers ::getLabel
   */
  public function testGetLabel(): void {
    $current_user = $this->prophesize(AccountProxyInterface::class);
    $external_roles = $this->prophesize(ExternalRolesInterface::class);
    $cache_context = new ExternalRolesCacheContext($current_user->reveal(), $external_roles->reveal());
    $this->assertEquals(new TranslatableMarkup('User External Roles'), $cache_context::getLabel());
  }

  /**
   * Tests the getContext method.
   *
   * @covers ::getContext
   */
  public function testGetContext(): void {
    $account = $this->prophesize(AccountInterface::class);
    $current_user = $this->prophesize(AccountProxyInterface::class);
    $current_user->getAccount()->willReturn($account->reveal());
    $external_roles = $this->prophesize(ExternalRolesInterface::class);
    $external_roles->getExternalRoles(Argument::any())->willReturn(['foo', 'bar', 'baz']);
    $cache_context = new ExternalRolesCacheContext($current_user->reveal(), $external_roles->reveal());
    $this->assertSame('foo,bar,baz', $cache_context->getContext());
  }

  /**
   * Tests the getCacheableMetadata method.
   *
   * @covers ::getCacheableMetadata
   */
  public function testGetCacheableMetadata(): void {
    $account = $this->prophesize(AccountInterface::class);
    $account->willImplement(CacheableDependencyInterface::class);
    $account->getCacheMaxAge()->willReturn(1);
    $account->getCacheContexts()->willReturn(['user.external_roles']);
    $account->getCacheTags()->willReturn(['foo,bar,baz']);
    $current_user = $this->prophesize(AccountProxyInterface::class);
    $current_user->getAccount()->willReturn($account->reveal());
    $external_roles = $this->prophesize(ExternalRolesInterface::class);
    $cache_context = new ExternalRolesCacheContext($current_user->reveal(), $external_roles->reveal());
    $expected = [
      'cacheContexts' => ['user.external_roles'],
      'cacheTags' => ['foo,bar,baz'],
      'cacheMaxAge' => 1,
    ];
    $this->assertEqualsCanonicalizing($expected, (array) $cache_context->getCacheableMetadata());
  }

}
