<?php

declare(strict_types=1);

namespace Drupal\Tests\external_roles\Unit;

use Drupal\Core\Site\Settings;
use Drupal\external_roles\ExternalRolesRepository;
use Drupal\external_roles\ExternalRolesRepositoryInterface;
use Drupal\Tests\UnitTestCase;

/**
 * Provides tests for the external roles repository.
 *
 * @coversDefaultClass \Drupal\external_roles\ExternalRolesRepository
 * @group external_roles
 */
class ExternalRolesRepositoryTest extends UnitTestCase {

  /**
   * The external roles repository to test.
   */
  protected ExternalRolesRepositoryInterface $repository;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    new Settings([
      'external_roles' => [
        'epsilon' => [
          'name' => 'Epsilon',
          'permissions' => [],
        ],
        'alpha' => [
          'name' => 'Alpha',
          'permissions' => [
            'access foo',
          ],
        ],
        'beta' => [
          'name' => 'Beta',
          'permissions' => [
            'access foo',
            'access bar',
            'access baz',
          ],
        ],
      ],
    ]);
    $this->repository = new ExternalRolesRepository();
  }

  /**
   * Tests the getPermissionsByRole method.
   *
   * @param string $role
   *   The external role.
   * @param array $expected_permissions
   *   The expected roles for the given role.
   *
   * @covers ::getPermissionsByRole
   * @covers ::loadDefinition
   * @dataProvider getPermissionsByRoleProvider
   */
  public function testGetPermissionsByRole(string $role, array $expected_permissions): void {
    $this->assertSame($expected_permissions, $this->repository->getPermissionsByRole($role));
    // Confirm that definition is statically cached. The extension list is
    // prophesized to get the path once.
    $this->assertSame($expected_permissions, $this->repository->getPermissionsByRole($role));
  }

  /**
   * Data provider for testGetPermissionsByRole.
   *
   * @return array
   *   A list of test scenarios.
   */
  public static function getPermissionsByRoleProvider(): array {

    $cases['unknown-role'] = [
      'role' => 'null',
      'expected_permissions' => [],
    ];

    $cases['no-permissions'] = [
      'role' => 'epsilon',
      'expected_permissions' => [],
    ];

    $cases['one-permission'] = [
      'role' => 'alpha',
      'expected_permissions' => [
        'access foo',
      ],
    ];

    $cases['beta-multiple-permissions'] = [
      'role' => 'beta',
      'expected_permissions' => [
        'access foo',
        'access bar',
        'access baz',
      ],
    ];

    return $cases;
  }

}
