<?php

declare(strict_types=1);

namespace Drupal\Tests\external_roles\Unit;

use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\ConditionInterface;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Database\Schema;
use Drupal\Core\Database\StatementInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\external_roles\ExternalRolesRepositoryInterface;
use Drupal\external_roles\Service\ExternalRoles;
use Drupal\external_roles\Service\ExternalRolesInterface;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Prophecy\Prophecy\ObjectProphecy;

/**
 * Provides tests for the external roles service.
 *
 * @coversDefaultClass \Drupal\external_roles\Service\ExternalRoles
 * @group external_roles
 */
class ExternalRolesTest extends UnitTestCase {

  /**
   * The external roles service to test.
   */
  protected ExternalRolesInterface $externalRoles;

  /**
   * The mocked database connection.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy<Connection>
   */
  protected ObjectProphecy $database;

  /**
   * The mocked external roles repository.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy<ExternalRolesRepositoryInterface>
   */
  protected ObjectProphecy $repository;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $schema = $this->prophesize(Schema::class);
    $schema->tableExists(Argument::any())->willReturn(TRUE);
    $this->database = $this->prophesize(Connection::class);
    $this->database->schema()->willReturn($schema);
    $this->repository = $this->prophesize(ExternalRolesRepositoryInterface::class);
    $this->externalRoles = new ExternalRoles($this->database->reveal(), $this->repository->reveal());
  }

  /**
   * Tests the getExternalRoles method.
   *
   * @param array $roles
   *   The external roles.
   *
   * @covers ::__construct
   * @covers ::getExternalRoles
   * @dataProvider getExternalRolesProvider
   */
  public function testGetExternalRoles(array $roles): void {
    $this->prophesizeDatabase($roles, 2);
    $account = $this->prophesize(AccountInterface::class);
    $account->id()->shouldBeCalled()->willReturn(2);
    $this->assertSame($roles, $this->externalRoles->getExternalRoles($account->reveal()));
    // Confirm that roles are statically cached. The prophecy is that the select
    // on the database only is called once.
    $this->assertSame($roles, $this->externalRoles->getExternalRoles($account->reveal()));
  }

  /**
   * Data provider for testGetExternalRoles.
   *
   * @return array
   *   A list of test scenarios.
   */
  public static function getExternalRolesProvider(): array {

    $cases['no-roles'] = [
      'roles' => [],
    ];

    $cases['one-role'] = [
      'roles' => [
        'foo',
      ],
    ];

    $cases['multiple-roles'] = [
      'roles' => [
        'foo',
        'bar',
        'baz',
      ],
    ];

    return $cases;
  }

  /**
   * Tests the getPermissions method.
   *
   * @param array $repository_permissions
   *   The permissions subsequently provided by the repository.
   * @param array $roles
   *   The external roles.
   * @param array $expected_permissions
   *   The expected permissions.
   *
   * @covers ::getPermissions
   * @dataProvider getPermissionsProvider
   */
  public function testGetPermissions(array $repository_permissions, array $roles, array $expected_permissions): void {
    $this->prophesizeDatabase($roles, 2);
    $this->repository->getPermissionsByRole(Argument::any())->willReturn(...$repository_permissions);
    $this->repository->reveal();
    $account = $this->prophesize(AccountInterface::class);
    $account->id()->shouldBeCalled()->willReturn(2);
    $this->assertSame($expected_permissions, $this->externalRoles->getPermissions($account->reveal()));
  }

  /**
   * Data provider for testGetPermissions.
   *
   * @return array
   *   A list of test scenarios.
   */
  public static function getPermissionsProvider(): array {

    $cases['empty-repository'] = [
      'repository_permissions' => [
        [],
        [],
        [],
      ],
      'roles' => [
        'alpha',
        'beta',
        'gamma',
      ],
      'expected_permissions' => [],
    ];

    $cases['no-roles'] = [
      'repository_permissions' => [
        ['access foo'],
        ['access bar'],
      ],
      'roles' => [],
      'expected_permissions' => [],
    ];

    $cases['one-role'] = [
      'repository_permissions' => [
        ['access foo'],
      ],
      'roles' => [
        'alpha',
      ],
      'expected_permissions' => [
        'access foo',
      ],
    ];

    $cases['multiple-roles'] = [
      'repository_permissions' => [
        ['access foo'],
        ['access bar', 'access xyz'],
        ['access baz'],
      ],
      'roles' => [
        'alpha',
        'beta',
        'gamma',
      ],
      'expected_permissions' => [
        'access foo',
        'access bar',
        'access xyz',
        'access baz',
      ],
    ];

    return $cases;
  }

  /**
   * Gets a database prophecy for the test cases.
   *
   * @param array $roles
   *   The external roles.
   * @param int $uid
   *   The account ID.
   */
  protected function prophesizeDatabase(array $roles, int $uid): void {

    $statement = $this->prophesize(StatementInterface::class);
    $statement->fetchCol()->shouldBeCalledOnce()->willReturn($roles);

    $select_three = $this->prophesize(SelectInterface::class);
    $select_three->execute()->shouldBeCalledOnce()->willReturn($statement->reveal());

    $select_two = $this->prophesize(SelectInterface::class);
    $select_two->orderBy(Argument::is('external_roles_value'))
      ->shouldBeCalledOnce()->willReturn($select_three->reveal());

    $condition = $this->prophesize(ConditionInterface::class);
    $condition->condition(Argument::is('r.entity_id'), Argument::is($uid))
      ->shouldBeCalledOnce()->willReturn($select_two->reveal());

    $select_one = $this->prophesize(SelectInterface::class);
    $select_one->fields(Argument::is('r'), Argument::is(['external_roles_value']))
      ->shouldBeCalledOnce()->willReturn($condition->reveal());

    $this->database->select(Argument::is('user__external_roles'), Argument::is('r'))
      ->shouldBeCalledOnce()->willReturn($select_one->reveal());
    $this->database->reveal();
  }

}
